@@ -6,28 +6,50 @@ package CGI::Application::Plugin::Config::Any;
 use strict;
 use warnings;
 
+use base 'Exporter';
+use vars '@EXPORT';
+@EXPORT = qw( config config_init config_name config_section config_read );
+
+use Config::Any;
+
+my $prefix = '__CONFIG_ANY_';
+
+$CGI::Application::Plugin::Config::Any::DEBUG = 0;
+
+
 =head1 NAME
 
 CGI::Application::Plugin::Config::Any - Add Config::Any Support to CGI::Application
 
 =head1 VERSION
 
-Version 0.04
+Version 0.10
 
 =cut
 
-$CGI::Application::Plugin::Config::Any::VERSION = '0.04';
-$CGI::Application::Plugin::Config::Any::DEBUG   = 0;
+$CGI::Application::Plugin::Config::Any::VERSION = '0.10';
 
-use base 'Exporter';
-use vars '@EXPORT';
-@EXPORT = qw( config );
-
-use Config::Any;
 
 =head1 SYNOPSIS
 
-In your L<CGI::Application|CGI::Application>-based module:
+There are two ways to initialize this module.
+
+B<In your instance script:>
+
+    my $app = WebApp->new(
+        PARAMS => {
+            config_dir    => '/path/to/configfiles',
+            config_files  => [ 'app.conf' ],
+            config_name   => 'main',
+            config_params => {
+                ## passed to Config::Any->load_files;
+                ## see Config::Any for valid params
+            }
+        }
+    );
+    $app->run();
+    
+B<In your L<CGI::Application|CGI::Application>-based module:>
 
     use base 'CGI::Application';
     use CGI::Application::Plugin::Config::Any;
@@ -36,11 +58,11 @@ In your L<CGI::Application|CGI::Application>-based module:
         my $self = shift;
 
         # Set config file and other options
-        $self->config->init(
-            configdir => '/path/to/configfiles',
-            files     => [ 'app.conf' ],
-            name      => 'main',
-            params    => {
+        $self->config_init(
+            config_dir    => '/path/to/configfiles',
+            config_files  => [ 'app.conf' ],
+            config_name   => 'main',
+            config_params => {
                 ## passed to Config::Any->load_files;
                 ## see Config::Any for valid params
             }
@@ -50,10 +72,10 @@ In your L<CGI::Application|CGI::Application>-based module:
 Later...
 
     ## get a complete config section as a hashref
-    $self->config->section( 'sectionname' );
+    my $section = $self->config_section( 'sectionname' );
     
     ## get a single config param
-    $self->config->param( 'sectionname.paramname' );
+    my $param = $self->config( 'paramname' );
     
 
 =head1 DESCRIPTION
@@ -68,84 +90,143 @@ to load configuration data from multiple different file formats. It
 supports XML, YAML, JSON, Apache-style configuration, Windows INI 
 files, and even Perl code.)
 
-=cut
 
+=head1 EXPORTS
 
-#-------------------------------------------------------------------
-# METHOD:     _new
-# + author:   Bianka Martinovic
-# + reviewed: Bianka Martinovic
-# + purpose:  internal constructor (non public method)
-#-------------------------------------------------------------------
-sub _new {
-    my $caller = shift;
-    my $class  = ref($caller) || $caller;
+=over 4
 
-    my %args   = ( @_ );
+=item config
+
+=item config_init
+
+=item config_name
+
+=item config_section
+
+=item config_read
+
+=back
 
-    my $set = {
-        '__CONFIG_ANY_DIR'   => delete $args{'configdir'},
-        '__CONFIG_ANY_FILES' => delete $args{'files'},
-        '__CONFIG_ANY_NAME'  => delete $args{'name'},
-    };
-        
-    my $self = bless $set, $class;
-    
-    return $self;
-    
-}   # --- end sub _new ---
 
 =head1 METHODS
 
-=head2 init
+=head2 config
+
+This method is exported to your C::A based application as an accessor
+to the configuration params.
+
+There are several ways to retrieve a config param:
+
+    $self->config_section('mysection');
+    $self->config('mysetting');
+    # set section to 'mysection' before retrieving 'mysetting'
+
+    $self->config('mysetting', section => 'mysection' );
+    # more convenient way to do the same as above
+
+    $self->config('mysection.mysetting');
+    # another way to do the same as above
+
+    $self->config('mysetting');
+    # let the module find a param named 'mysetting' without
+    # knowing or bothering the section name
+
+See L<bugs_caveats|BUGS/CAVEATS>!
+
+=cut
+
+#-------------------------------------------------------------------
+# METHOD:     config
+# + author:   Bianka Martinovic
+# + reviewed: Bianka Martinovic
+# + purpose:
+#-------------------------------------------------------------------
+sub config {
+    my $self    = shift;
+    my $param   = shift;
+
+    my %attrs = (
+        section => $self->{$prefix.'CURRENT_SECTION'},
+        name    => $self->{$prefix.'CONFIG_NAME'} || 'default',
+        @_
+    );
+
+    my $section = $attrs{'section'};
+
+    if ( $param =~ /^(.*)\.(.*)$/ ) {
+        $section = $1;
+        $param   = $2;
+    }
+
+    $CGI::Application::Plugin::Config::Any::DEBUG
+        and __PACKAGE__->_debug(
+              "    config name [$attrs{'name'}]\n"
+            . "          param [$param]\n"
+            . "        section [$section]\n"
+        );
+
+    return _load(
+        $self,
+        section => $section,
+        param   => $param,
+        name    => $attrs{'name'}
+    );
+}   # --- end sub config ---
+
+
+=head2 config_init
 
 Initializes the plugin.
 
-    $self->config->init(
-        configdir => '/path/to/configfiles',
-        files     => [ 'app.conf' ],
+    $self->config_init(
+        config_dir   => '/path/to/configfiles',
+        config_files => [ 'app.conf' ],
     );
 
 Valid params:
 
 =over 4
 
-=item configdir SCALAR
+=item config_dir SCALAR
 
 Path where the config files reside in.
 
-=item files ARRAY
+=item config_files ARRAY
 
 A list of files to load.
 
-=item name SCALAR
+=item config_name SCALAR
 
 You can use more than one configuration at the same time by using config
 names. For example:
 
-    $self->config->init(
-        name   => 'database',
-        files  => [ 'db.conf' ],
+    $self->config_init(
+        config_name   => 'database',
+        config_files  => [ 'db.conf' ],
     );
-    $self->config->init(
-        name   => 'template',
-        files  => [ 'tpl.conf' ],
+    $self->config_init(
+        config_name   => 'template',
+        config_files  => [ 'tpl.conf' ],
     );
 
     ...
 
-    my $connection_options  = $self->config('database')->section('connection');
-    my $template_config     = $self->config('template')->param('file');
+    my $connection_options  = $self->config_section('connection', name => 'database' );
+    my $template_file       = $self->config( 'file', name => 'template' );
 
-=item params HASHREF
+=item config_names HASHREF
+
+
+
+=item config_params HASHREF
 
 Options to pass to Config::Any->load_files().
 
 B<Example:>
 
-    $self->config->init(
-        files  => [ 'default.yml' ],
-        params => {
+    $self->config_init(
+        config_files  => [ 'default.yml' ],
+        config_params => {
             'use_ext' => 1,
         }
     );
@@ -157,190 +238,124 @@ See L<Config::Any> for details.
 =cut
 
 #-------------------------------------------------------------------
-# METHOD:     init
+# METHOD:     config_init
 # + author:   Bianka Martinovic
 # + reviewed: Bianka Martinovic
 # + purpose:  
 #-------------------------------------------------------------------
-sub init {
+sub config_init {
     my $self = shift;
-    
+
     my %args = (
-        'name'      => $self->{'__CONFIG_ANY_NAME'}  || ref $self,
-        'configdir' => $self->{'__CONFIG_ANY_DIR'}   || undef,
-        'files'     => $self->{'__CONFIG_ANY_FILES'} || undef,
-        'params'    => {},
+        'config_names'  => $self->param('config_names')  || {},
+        'config_dir'    => $self->param('config_dir')    || undef,
+        'config_files'  => $self->param('config_files')  || [],
+        'config_params' => $self->param('config_params') || {},
+        'config_name'   => $self->param('config_name')   || 'default',
         @_
     );
     
-    $self->{'__CONFIG_ANY_DIR'}    = delete $args{'configdir'};
-    $self->{'__CONFIG_ANY_FILES'}  = delete $args{'files'};
-    $self->{'__CONFIG_ANY_NAME'}   = delete $args{'name'};
-    $self->{'__CONFIG_ANY_PARAMS'} = delete $args{'params'};
-    
+    foreach ( keys %args ) {
+        $self->{ $prefix . uc($_) } = delete $args{$_};
+    }
+
     $CGI::Application::Plugin::Config::Any::DEBUG
-        and $self->_debug(
+        and __PACKAGE__->_debug(
               "initialized with:\n"
-            . "\tname:      $self->{'__CONFIG_ANY_NAME'}\n"
-            . "\tconfigdir: $self->{'__CONFIG_ANY_DIR'}\n"
-            . "\tfiles:     "
-            . join( ', ', @{ $self->{'__CONFIG_ANY_FILES'} } )
+            . "\tconfig_names: $self->{$prefix.'CONFIG_NAMES'}\n"
+            . "\tconfig_dir  : $self->{$prefix.'CONFIG_DIR'}\n"
+            . "\tconfig_files: "
+            . join( ', ', @{ $self->{$prefix.'CONFIG_FILES'} } )
         );
 
     return 1;
     
-}   # --- end sub init ---
+}   # --- end sub config_init ---
 
+=head2 config_name
 
-=head2 config
-
-This method is exported to your C::A based application as an accessor
-to the configuration methods.
+Set the name of the config to use.
 
 =cut
 
 #-------------------------------------------------------------------
-# METHOD:     config
+# METHOD:     config_name
 # + author:   Bianka Martinovic
 # + reviewed: Bianka Martinovic
-# + purpose:  
+# + purpose:  set the name of the current config
 #-------------------------------------------------------------------
-sub config {
+sub config_name {
     my $self = shift;
-    my $conf = shift || ref $self;
-
-    if ( $conf ) {
+    my $name = shift;
     
-        if ( ! exists $self->{'__CONFIG_ANY_LOADED'}->{ $conf } ) {
-        
-            $self->{'__CONFIG_ANY_LOADED'}->{ $conf } 
-                = CGI::Application::Plugin::Config::Any->_new( 
-                    'name' => $conf, 
-                    @_ 
-                );
-        }
-        
-        return $self->{'__CONFIG_ANY_LOADED'}->{ $conf };
-        
-    }    
-
-    return CGI::Application::Plugin::Config::Any->_new(
-        'name' => ref $self, 
-        @_
-    );
-
-}   # --- end sub config ---
-
-
-=head2 param
-
-Retrieve a value from your configuration.
-
-Examples:
-
-    $self->config->section('mysection')->param('mysetting');
-    # set the section to 'mysection' before retrieving 'mysetting'
-    
-    $self->config->param('mysetting','mysection');
-    # more convenient way to do the same as above
-    
-    $self->config->param('mysection.mysetting');
-    # another way to do the same as above
-    
-    $self->config->param('mysetting');
-    # if no section name is given, the name of the last section
-    # named by ->section() or ->param(<section>.<attribute>) syntax
-    # is used; this may change in future, so don't rely on it!
-
-=cut
-
-#-------------------------------------------------------------------
-# METHOD:     param
-# + author:   Bianka Martinovic
-# + reviewed: Bianka Martinovic
-# + purpose:  
-#-------------------------------------------------------------------
-sub param {
-    my $self    = shift;
-    my $param   = shift;
-    my $section = shift;
-    
-    if ( $param =~ /^(.*)\.(.*)$/ ) {
-        $section = $1;
-        $param   = $2;
-    }
-    
-    if ( ! $section && $self->{'__CURRENT_SECTION'} ) {
-        $section = $self->{'__CURRENT_SECTION'};
-    }
+    return unless $name;
     
     $CGI::Application::Plugin::Config::Any::DEBUG
-        and $self->_debug(
-              "\nCGI::Application::Plugin::Config::Any\n"
-            . "    loading param [$param]\n"
-            . "          section [$section]\n"
-        );
+        and __PACKAGE__->_debug( "setting config name: $name" );
     
-    return $self->_load(
-        section => $section,
-        param   => $param
-    );
+    $self->{$prefix.'CONFIG_NAME'} = $name;
     
-}   # --- end sub param ---
+    return $name;
+
+}   # --- end sub config_name ---
 
 
-=head2 section
+=head2 config_section
 
 Retrieve a complete section from your configuration, or set the name
-of the current "default section" for later use with ->param().
+of the current "default section" for later use with C<config()>.
 
-    my $hash = $self->config->section('mysection');
+    my $hash = $self->config_section('mysection');
 
 =cut
 
 #-------------------------------------------------------------------
-# METHOD:     section
+# METHOD:     config_section
 # + author:   Bianka Martinovic
 # + reviewed: Bianka Martinovic
 # + purpose:  
 #-------------------------------------------------------------------
-sub section {
+sub config_section {
     my $self    = shift;
     my $section = shift;
     
-    $self->{'__CURRENT_SECTION'} = $section;
+    $self->{$prefix.'CURRENT_SECTION'} = $section;
     
     $CGI::Application::Plugin::Config::Any::DEBUG
-        and $self->_debug(
+        and __PACKAGE__->_debug(
             "loading section [$section]"
         );
     
-    return $self->_load( section => $section ) if defined wantarray;
+    return _load( $self, section => $section, @_ ) if defined wantarray;
     
     return;
     
-}   # --- end sub section ---
+}   # --- end sub config_section ---
 
 
-=head2 getall
+=head2 config_read
 
 Get complete configuration as a hashref.
 
+    my $config = $self->config_read();
+
 =cut
 
 #-------------------------------------------------------------------
-# METHOD:     getall
+# METHOD:     config_read
 # + author:   Bianka Martinovic
 # + reviewed: Bianka Martinovic
 # + purpose:  
 #-------------------------------------------------------------------
-sub getall {
+sub config_read {
     my $self = shift;
     
-    return $self->_load();
+    return _load( $self, @_ );
     
-}   # --- end sub getall ---
+}   # --- end sub config_read ---
 
+# required by C::A::Standard::Config;
+sub std_config { return 1; }
 
 #-------------------------------------------------------------------
 #               + + + + + INTERNAL METHODS + + + + +
@@ -354,40 +369,74 @@ sub getall {
 #-------------------------------------------------------------------
 sub _load {
     my $self = shift;
-    
+
     my %args = (
         section   => undef,
         param     => undef,
+        name      => $self->{$prefix.'CONFIG_NAME'} || 'default',
         @_
     );
-      
-    my %config = ();    
+    
+    my $name = $args{'name'};
+
+    unless ( $self->{$prefix.'CONFIG_CONFIG'} ) {
+        $self->config_init( %args );
+    }
+
+    my %config = ();
     
     ## config already loaded?
-    unless ( $self->{'__CONFIG_ANY_CONFIG'}->{ $self->{'__CONFIG_ANY_NAME'} } ) {
+    unless ( $self->{$prefix.'CONFIG_CONFIG'}->{ $name } ) {
     
         $CGI::Application::Plugin::Config::Any::DEBUG
-            and $self->_debug(
-                "loading config named [$self->{'__CONFIG_ANY_NAME'}]"
+            and __PACKAGE__->_debug(
+                "loading config named [$name]"
             );
 
-        if ( $self->{'__CONFIG_ANY_FILES'} && ref $self->{'__CONFIG_ANY_FILES'} ne 'ARRAY' ) {
-            $self->{'__CONFIG_ANY_FILES'} = [ $self->{'__CONFIG_ANY_FILES'} ];
+        if ( exists $self->{$prefix.'CONFIG_NAMES'}->{ $name } ) {
+        
+            my $this = $self->{$prefix.'CONFIG_NAMES'}->{ $name };
+
+            foreach ( qw/ config_dir config_files config_params/ ) {
+                my $key = $prefix.uc($_);
+                if ( exists $this->{ $_ } ) {
+                    $self->{$key} = $this->{ $_ };
+                }
+                $self->{$prefix.'CONFIG_FILES'}
+                    = $self->{$prefix.'CONFIG_NAMES'}->{ $name }->{'config_files'};
+            }
+        }
+
+        if ( $self->{$prefix.'CONFIG_FILES'}
+          && ref $self->{$prefix.'CONFIG_FILES'} ne 'ARRAY'
+        ) {
+            $self->{$prefix.'CONFIG_FILES'} = [ $self->{$prefix.'CONFIG_FILES'} ];
         }
         
-        $self->{'__CONFIG_ANY_FILES'} 
+        $self->{$prefix.'CONFIG_FILES'}
             = [ 
-                map { $self->{'__CONFIG_ANY_DIR'}.'/'.$self->{'__CONFIG_ANY_FILES'}[$_] } 
-                    0 .. $#{ $self->{'__CONFIG_ANY_FILES'} } 
+                map { $self->{$prefix.'CONFIG_DIR'}.'/'.$self->{$prefix.'CONFIG_FILES'}[$_] }
+                    0 .. $#{ $self->{$prefix.'CONFIG_FILES'} }
               ];
-    
+
+        $CGI::Application::Plugin::Config::Any::DEBUG
+            and __PACKAGE__->_debug(
+                "searching files: "
+              . join( ', ', @{$self->{$prefix.'CONFIG_FILES'}} )
+            );
+
         ## load the files using Config::Any
         my $cfg = Config::Any->load_files( 
                       { 
-                          files   => $self->{'__CONFIG_ANY_FILES'}, 
-                          %{ $self->{'__CONFIG_ANY_PARAMS'} }
+                          files   => $self->{$prefix.'CONFIG_FILES'},
+                          %{ $self->{$prefix.'CONFIG_PARAMS'} }
                       }
                   );
+
+        $CGI::Application::Plugin::Config::Any::DEBUG
+            and __PACKAGE__->_debug(
+                "found [" . scalar @$cfg . "] config files"
+            );
     
         ## import settings
         for ( @$cfg ) {
@@ -400,18 +449,18 @@ sub _load {
         
         }
     
-        $self->{'__CONFIG_ANY_CONFIG'}->{ $self->{'__CONFIG_ANY_NAME'} } = \%config;
+        $self->{$prefix.'CONFIG_CONFIG'}->{ $args{'name'} } = \%config;
         
     }
     else {
-        %config = %{ $self->{'__CONFIG_ANY_CONFIG'}->{ $self->{'__CONFIG_ANY_NAME'} } };
+        %config = %{ $self->{$prefix.'CONFIG_CONFIG'}->{ $args{'name'} } };
     }
 
     ## return a section
     if ( $args{'section'} && ! $args{'param'} ) {
     
         $CGI::Application::Plugin::Config::Any::DEBUG
-            and $self->_debug(
+            and __PACKAGE__->_debug(
                 "returning complete section [$args{'section'}]"
             );
     
@@ -426,10 +475,10 @@ sub _load {
                  
         unless ( defined $value ) {
             $CGI::Application::Plugin::Config::Any::DEBUG
-                and $self->_debug(
+                and __PACKAGE__->_debug(
                     "trying to find key [$args{'param'}]"
                 );
-            $value = $self->_find_key( $args{'param'}, \%config );
+            $value = _find_key( $self, $args{'param'}, \%config );
         }
 
         return $value;
@@ -437,7 +486,7 @@ sub _load {
     }
 
     return \%config;# unless wantarray;
-    
+
 }   # --- end sub _load ---
 
 #-------------------------------------------------------------------
@@ -460,7 +509,7 @@ sub _find_key {
     }
     
     foreach my $subkey ( keys %{ $config } ) {
-        my $value = $self->_find_key( $key, $config->{$subkey} );
+        my $value = _find_key( $self, $key, $config->{$subkey} );
         return $value if $value;
     }
     
@@ -574,7 +623,7 @@ Anyway, if CAP::Config::Any isn't able to find a required key in the current
 section, it walks through the complete config data structure to find it. So,
 the following works with this example:
 
-    my $param = $self->config->param('ShowErrorStatement');
+    my $param = $self->config('ShowErrorStatement');
     ## this will return '1'
 
 There is no way to suppress this at the moment, so beware of having similar
